@@ -1,9 +1,11 @@
+import atexit
+
 from .exceptions import ConnectionError, TimeoutError, PacketError
 from .heartbeats import HeartbeatThread
 from .logs import LoggingMixin
 from .namespaces import (
-    EngineIONamespace, SocketIONamespace, LoggingSocketIONamespace,
-    find_callback)
+    EngineIONamespace, SocketIONamespace,
+    LoggingSocketIONamespace, find_callback, make_logging_prefix)
 from .parsers import (
     parse_host, parse_engineIO_session,
     format_socketIO_packet_data, parse_socketIO_packet_data,
@@ -14,7 +16,7 @@ from .transports import (
 
 
 __all__ = 'SocketIO', 'SocketIONamespace'
-__version__ = '0.6.6'
+__version__ = '0.7.0'
 BaseNamespace = SocketIONamespace
 LoggingNamespace = LoggingSocketIONamespace
 
@@ -43,8 +45,9 @@ class EngineIO(LoggingMixin):
         self._http_session = prepare_http_session(kw)
 
         self._log_name = self._url
-        self._wants_to_close = False
         self._opened = False
+        self._wants_to_close = False
+        atexit.register(self._close)
 
         if Namespace:
             self.define(Namespace)
@@ -75,7 +78,8 @@ class EngineIO(LoggingMixin):
             except (TimeoutError, ConnectionError) as e:
                 if not self._wait_for_connection:
                     raise
-                warning = Exception('[waiting for connection] %s' % e)
+                warning = Exception(
+                    '[engine.io waiting for connection] %s' % e)
                 warning_screen.throw(warning)
         assert engineIO_packet_type == 0  # engineIO_packet_type == open
         return parse_engineIO_session(engineIO_packet_data)
@@ -98,7 +102,7 @@ class EngineIO(LoggingMixin):
                         self._warn('unexpected engine.io packet')
             except Exception:
                 pass
-        self._debug('[transport selected] %s', self.transport_name)
+        self._debug('[engine.io transport selected] %s', self.transport_name)
 
     def _reset_heartbeat(self):
         try:
@@ -120,7 +124,7 @@ class EngineIO(LoggingMixin):
         self._heartbeat_thread.start()
         if hurried:
             self._heartbeat_thread.hurry()
-        self._debug('[heartbeat reset]')
+        self._debug('[engine.io heartbeat reset]')
 
     def _connect_namespaces(self):
         pass
@@ -247,6 +251,9 @@ class EngineIO(LoggingMixin):
                     self._process_packets()
                 except TimeoutError:
                     pass
+                except KeyboardInterrupt:
+                    self._close()
+                    raise
             except ConnectionError as e:
                 self._opened = False
                 try:
@@ -256,7 +263,7 @@ class EngineIO(LoggingMixin):
                     self._warn(warning)
                 try:
                     namespace = self.get_namespace()
-                    namespace.on_disconnect()
+                    namespace._find_packet_callback('disconnect')()
                 except PacketError:
                     pass
         self._heartbeat_thread.relax()
@@ -290,7 +297,7 @@ class EngineIO(LoggingMixin):
             raise PacketError(
                 'unexpected engine.io packet type (%s)' % engineIO_packet_type)
         delegate(engineIO_packet_data, namespace)
-        if engineIO_packet_type is 4:
+        if engineIO_packet_type == 4:
             return engineIO_packet_data
 
     def _on_open(self, data, namespace):
@@ -371,7 +378,7 @@ class SocketIO(EngineIO):
         self._namespace_by_path[path] = namespace = Namespace(self, path)
         if path:
             self.connect(path)
-            self.wait(for_connect=True)
+            self.wait(for_namespace=namespace)
         return namespace
 
     def on(self, event, callback, path=''):
@@ -389,26 +396,30 @@ class SocketIO(EngineIO):
 
     # Act
 
-    def connect(self, path, with_transport_instance=False):
-        socketIO_packet_type = 0
-        socketIO_packet_data = format_socketIO_packet_data(path)
-        self._message(
-            str(socketIO_packet_type) + socketIO_packet_data,
-            with_transport_instance)
+    def connect(self, path='', with_transport_instance=False):
+        if path or not self.connected:
+            socketIO_packet_type = 0
+            socketIO_packet_data = format_socketIO_packet_data(path)
+            self._message(
+                str(socketIO_packet_type) + socketIO_packet_data,
+                with_transport_instance)
+        self._wants_to_close = False
 
     def disconnect(self, path=''):
-        if not path or not self._opened:
-            self._close()
-        elif path:
+        if path and self._opened:
             socketIO_packet_type = 1
             socketIO_packet_data = format_socketIO_packet_data(path)
             try:
                 self._message(str(socketIO_packet_type) + socketIO_packet_data)
             except (TimeoutError, ConnectionError):
                 pass
+        elif not path:
+            self._close()
         try:
-            namespace = self._namespace_by_path.pop(path)
-            namespace.on_disconnect()
+            namespace = self._namespace_by_path[path]
+            namespace._find_packet_callback('disconnect')()
+            if path:
+                del self._namespace_by_path[path]
         except KeyError:
             pass
 
@@ -438,13 +449,17 @@ class SocketIO(EngineIO):
     def wait_for_callbacks(self, seconds=None):
         self.wait(seconds, for_callbacks=True)
 
-    def _should_stop_waiting(self, for_connect=False, for_callbacks=False):
-        if for_connect:
-            for namespace in self._namespace_by_path.values():
-                is_namespace_connected = getattr(
-                    namespace, '_connected', False)
-                if not is_namespace_connected:
-                    return False
+    def _should_stop_waiting(self, for_namespace=False, for_callbacks=False):
+        if for_namespace:
+            namespace = for_namespace
+            if getattr(namespace, '_invalid', False):
+                raise ConnectionError(
+                    'invalid socket.io namespace (%s)' % namespace.path)
+            if not getattr(namespace, '_connected', False):
+                self._debug(
+                    '%s[socket.io waiting for connection]',
+                    make_logging_prefix(namespace.path))
+                return False
             return True
         if for_callbacks and not self._has_ack_callback:
             return True
@@ -473,19 +488,20 @@ class SocketIO(EngineIO):
         except KeyError:
             raise PacketError(
                 'unexpected socket.io packet type (%s)' % socketIO_packet_type)
-        delegate(socketIO_packet_data, namespace)
+        delegate(parse_socketIO_packet_data(socketIO_packet_data), namespace)
         return socketIO_packet_data
 
-    def _on_connect(self, data, namespace):
+    def _on_connect(self, data_parsed, namespace):
         namespace._connected = True
         namespace._find_packet_callback('connect')()
+        self._debug(
+            '%s[socket.io connected]', make_logging_prefix(namespace.path))
 
-    def _on_disconnect(self, data, namespace):
+    def _on_disconnect(self, data_parsed, namespace):
         namespace._connected = False
         namespace._find_packet_callback('disconnect')()
 
-    def _on_event(self, data, namespace):
-        data_parsed = parse_socketIO_packet_data(data)
+    def _on_event(self, data_parsed, namespace):
         args = data_parsed.args
         try:
             event = args.pop(0)
@@ -496,21 +512,20 @@ class SocketIO(EngineIO):
                 data_parsed.path, data_parsed.ack_id))
         namespace._find_packet_callback(event)(*args)
 
-    def _on_ack(self, data, namespace):
-        data_parsed = parse_socketIO_packet_data(data)
+    def _on_ack(self, data_parsed, namespace):
         try:
             ack_callback = self._get_ack_callback(data_parsed.ack_id)
         except KeyError:
             return
         ack_callback(*data_parsed.args)
 
-    def _on_error(self, data, namespace):
-        namespace._find_packet_callback('error')(data)
+    def _on_error(self, data_parsed, namespace):
+        namespace._find_packet_callback('error')(*data_parsed.args)
 
-    def _on_binary_event(self, data, namespace):
+    def _on_binary_event(self, data_parsed, namespace):
         self._warn('[not implemented] binary event')
 
-    def _on_binary_ack(self, data, namespace):
+    def _on_binary_ack(self, data_parsed, namespace):
         self._warn('[not implemented] binary ack')
 
     def _prepare_to_send_ack(self, path, ack_id):
